/**
 * Copyright (c) 2010 Perforce Software. All rights reserved.
 */
package com.perforce.maven.scm.provider.p4.command.status;

import java.util.List;

import org.apache.maven.scm.ScmException;
import org.apache.maven.scm.ScmFileSet;
import org.apache.maven.scm.command.status.AbstractStatusCommand;
import org.apache.maven.scm.command.status.StatusScmResult;
import org.apache.maven.scm.provider.ScmProviderRepository;

import com.perforce.maven.scm.provider.p4.command.P4Command;
import com.perforce.maven.scm.provider.p4.util.P4Utils;
import com.perforce.p4java.client.IClient;
import com.perforce.p4java.core.file.IFileSpec;
import com.perforce.p4java.exception.P4JavaException;
import com.perforce.p4java.option.server.OpenedFilesOptions;

/**
 * The class uses the Perforce opened command to list opened files and display file status.
 */
public class P4StatusCommand
    extends AbstractStatusCommand
    implements P4Command
{

    /** The Constant command. */
    public static final String command = "Perforce opened command";

    private IClient client;

    public void setClient( IClient client )
    {
        this.client = client;
    }

    /**
     * @see org.apache.maven.scm.command.status.AbstractStatusCommand
     *      executeStatusCommand(org.apache.maven.scm.provider.ScmProviderRepository, org.apache.maven.scm.ScmFileSet)
     */
    protected StatusScmResult executeStatusCommand( ScmProviderRepository repo, ScmFileSet files )
        throws ScmException
    {
        P4StatusResult p4Result = new P4StatusResult();

        List<IFileSpec> fileSpecs = P4Utils.scmFileSet2P4FileSpecs( files );

        if ( !fileSpecs.isEmpty() )
        {
            try
            {
                OpenedFilesOptions openedFilesOptions = new OpenedFilesOptions();
                List<IFileSpec> retFileSpecs = client.openedFiles( fileSpecs, openedFilesOptions );

                p4Result.process( retFileSpecs  );
            }
            catch ( P4JavaException e )
            {
                throw new ScmException( e.getLocalizedMessage(), e );
            }
        }

        if ( !p4Result.isSuccess() )
        {
            return new StatusScmResult( command, "The opened command failed.", p4Result.getCommandOutput(),
                                        p4Result.isSuccess() );
        }

        return new StatusScmResult( command, p4Result.getCommandResult() );
    }
}
